'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

// Inline polyfill for Object.assign so we can target ES5.
var objectAssign = function () {
    objectAssign = Object.assign || function objectAssign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s)
                if (Object.prototype.hasOwnProperty.call(s, p))
                    t[p] = s[p];
        }
        return t;
    };
    return objectAssign.apply(this, arguments);
};
/**
 * Inject the value of a single React context into a component's property at runtime,
 * while ensuring that users of the component can not override or even see the
 * property being injected into.
 *
 * This allows components to automatically map properties to context values,
 * while ensuring consumers of the component don't see properties that they
 * don't need to provide a value for.
 *
 * @param Context The React v16 context object to pull the value from.
 * @param PropName The name of the property to inject the value into.
 * @param WrappedComponent The component that is being wrapped.
 * @returns The component that should be used by consumers or exported.
 *
 * @example
 * // Import the InjectSingle function.
 * import { InjectSingle } from "react-injectable";
 *
 * // The React v16 context (can be imported from elsewhere).
 * export const MyContext = React.createContext<SomeType>(...);
 *
 * // The component properties (include injected properties here).
 * interface ExampleProps {
 *   someProp: string;
 *   injectedProp: SomeType;
 * }
 *
 * // The component declaration.
 * export const Example = InjectSingle(
 *   MyContext,
 *   "injectedProp",
 *   class Example extends React.Component<ExampleProps, {}> {
 *     constructor(props: ExampleProps) {
 *       super(props);
 *     }
 *   });
 *
 * // Using the component in code.
 * function usageExample() {
 *   return (
 *     <NavigationListener
 *       someProp="hello"
 *     />
 *   );
 * }
 */
function InjectSingle(Context, PropName, Component) {
    return React.forwardRef(function (props, ref) {
        return (React.createElement(Context.Consumer, null, function (value) {
            var _a;
            var injectedProps = objectAssign((_a = {},
                _a[PropName] = value,
                _a), props);
            return (React.createElement(Component, __assign({ ref: ref }, injectedProps)));
        }));
    });
}
/**
 * Inject the value of multiple React contexts into a set of component properties at runtime,
 * while ensuring that users of the component can not override or even see the
 * properties being injected into.
 *
 * This allows components to automatically map properties to context values,
 * while ensuring consumers of the component don't see properties that they
 * don't need to provide a value for.
 *
 * @param ContextMap A map of properties to React v16 context objects.
 * @param Component The component to inject properties into.
 * @returns The component that should be used by consumers or exported.
 *
 * @example
 * // Import the Inject function.
 * import { Inject } from "react-injectable";
 *
 * // The React v16 contexts (can be imported from elsewhere).
 * export const MyContext1 = React.createContext<SomeType>(...);
 * export const MyContext2 = React.createContext<string>("hello");
 *
 * // The component properties (include injected properties here).
 * interface ExampleProps {
 *   someProp: string;
 *   injectedProp: SomeType;
 *   anotherInjectedProp: string;
 * }
 *
 * // The component declaration.
 * export const Example = Inject(
 *   {
 *     injectedProp: MyContext1,
 *     anotherInjectedProp: MyContext2,
 *   },
 *   class Example extends React.Component<ExampleProps, {}> {
 *     constructor(props: ExampleProps) {
 *       super(props);
 *     }
 *   });
 *
 * // Using the component in code.
 * function usageExample() {
 *   return (
 *     <NavigationListener
 *       someProp="hello"
 *     />
 *   );
 * }
 */
function Inject(ContextMap, WrappedComponent) {
    // TODO: I don't think there's anyway to avoid the use of "any" here, since we can't
    // re-assign to "current" as we strip out each property iteratively.
    var current = WrappedComponent;
    for (var cv in ContextMap) {
        current = InjectSingle(ContextMap[cv], cv, current);
    }
    return current;
}

exports.InjectSingle = InjectSingle;
exports.Inject = Inject;
